# Copyright (c) 2009 gocept gmbh & co. kg
# See also LICENSE.txt

import yaml
import grok
import gocept.directory.interfaces
import zope.app.component.hooks
import os.path
import netaddr


class PaltryDirectory(grok.GlobalUtility):

    grok.implements(gocept.directory.interfaces.IPaltryDirectory)
    @property
    def _dir(self):
        site = zope.app.component.hooks.getSite()
        assert gocept.directory.interfaces.IDirectory.providedBy(site)
        return site.paltry

    @property
    def passwd_db(self):
        return os.path.join(self._dir, 'passwords')

    @property
    def people_db(self):
        return os.path.join(self._dir, 'people', 'people.yml')

    @property
    def groups_db(self):
        return os.path.join(self._dir, 'people', 'groups.yml')

    @property
    def location_db(self):
        return os.path.join(self._dir, 'locations')

    def get_password(self, service):
        if not service in os.listdir(self.passwd_db):
            raise KeyError(service)
        return open(os.path.join(self.passwd_db, service)).read().strip()

    def list_passwords(self):
        passwords = {}
        for service in os.listdir(self.passwd_db):
            if service.startswith('.'):
                continue
            passwords[service] = open(os.path.join(self.passwd_db, service)).read().strip()
        return passwords

    def list_people(self):
        return yaml.load(open(self.people_db))

    def list_groups(self):
        return yaml.load(open(self.groups_db))

    def list_locations(self):
        locations = [l for l in os.listdir(self.location_db)
                     if not l.startswith('.')]
        return sorted(locations)

    def lookup_networks(self, location, af=''):
        networks = yaml.load(
            open(os.path.join(self.location_db, location)))['networks']
        # replace inner dicts with lists
        for vlan, nets in networks.items():
            if af:
                filtered_nets = [val for key, val in nets.items() if key == af]
            else:
                filtered_nets = nets.values()
            if filtered_nets:
                networks[vlan] = sorted(filtered_nets)
            else:
                del networks[vlan]
        return networks

    def list_networks(self):
        # Networks may be used in multiple VLANs, we only list them once.
        networks = set()
        for location in os.listdir(self.location_db):
            if location.startswith('.'):
                continue
            for network in self.lookup_networks(location).values():
                networks.update(network)
        return sorted(networks)

    def _build_address(self, nodeparams, network, vlan):
        if network.addr_type == 6:
            return network[nodeparams['id']]
        elif network.addr_type == 4:
            try:
                return network[nodeparams['ip4_%s' % vlan]]
            except KeyError:
                pass

    def lookup_interfaces(self, nodename, af_filter=''):
        interfaces = {}
        nodes = zope.component.getUtility(gocept.directory.interfaces.INodes)
        node = nodes[nodename]
        params = node['parameters']
        for vlan, networks in self.lookup_networks(
                params['location'], af_filter).items():
            for n in networks:
                network = netaddr.CIDR(n)
                addr = self._build_address(params, network, vlan)
                if addr:
                    try:
                        addresses = interfaces[vlan]
                    except KeyError:
                        addresses = interfaces[vlan] = []
                    addresses.append('%s/%i' % (str(addr), network.prefixlen))
        return interfaces

    def lookup_addresses(self, nodename, vlan_filter='', af_filter=''):
        addresses = []
        for vlan, addr in self.lookup_interfaces(nodename, af_filter).items():
            if vlan_filter and vlan != vlan_filter:
                continue
            addresses.extend(addr)
        return addresses
