# Copyright (c) 2009 gocept gmbh & co. kg
# See also LICENSE.txt

import grok
import gocept.directory.interfaces
import gocept.reference
import megrok.pagelet


class VLANClass(grok.Model):
    """All locations are supposed to have the same set of VLANs.

    This is what is described by a VLAN class.

    """
    grok.implements(gocept.directory.interfaces.IVLANClass)


class AddVLANClass(gocept.directory.form.BasicAddForm):

    factory = VLANClass

    grok.context(gocept.directory.interfaces.IDirectory)
    form_fields = grok.AutoFields(gocept.directory.interfaces.IVLANClass)


class VLAN(grok.Container):
    """A specific VLAN in a location."""

    grok.implements(gocept.directory.interfaces.IVLAN)

    class_ = gocept.reference.Reference(ensure_integrity=True)

    @property
    def location(self):
        return self.__parent__

    @property
    def networks(self):
        for obj in self.values():
            if gocept.directory.interfaces.INetwork.providedBy(obj):
                yield obj


class Index(megrok.pagelet.Pagelet):

    grok.context(VLAN)
    grok.require('gocept.directory.ManageDirectory')


@grok.subscribe(gocept.directory.interfaces.IVLANClass, grok.ObjectAddedEvent)
def add_new_vlan_to_locations(vlan_class, event):
    directory = vlan_class.__parent__
    for location in directory.locations:
        vlan = VLAN()
        vlan.class_ = vlan_class
        location['vlan-%s' % vlan_class.short] = vlan


@grok.subscribe(gocept.directory.interfaces.ILocation, grok.ObjectAddedEvent)
def add_vlans_to_new_location(location, event):
    directory = location.__parent__
    for vlan_class in directory.vlan_classes:
        vlan = VLAN()
        vlan.class_ = vlan_class
        location['vlan-%s' % vlan_class.short] = vlan


class VLANClasses(megrok.pagelet.Pagelet):
    grok.context(gocept.directory.interfaces.IDirectory)
    grok.template('classes')
    grok.require('gocept.directory.ManageDirectory')
